<?php
/**
 * Jamroom 5 Search module
 *
 * copyright 2003 - 2015
 * by The Jamroom Network
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0.  Please see the included "license.html" file.
 *
 * This module may include works that are not developed by
 * The Jamroom Network
 * and are used under license - any licenses are included and
 * can be found in the "contrib" directory within this module.
 *
 * Jamroom may use modules and skins that are licensed by third party
 * developers, and licensed under a different license  - please
 * reference the individual module or skin license that is included
 * with your installation.
 *
 * This software is provided "as is" and any express or implied
 * warranties, including, but not limited to, the implied warranties
 * of merchantability and fitness for a particular purpose are
 * disclaimed.  In no event shall the Jamroom Network be liable for
 * any direct, indirect, incidental, special, exemplary or
 * consequential damages (including but not limited to, procurement
 * of substitute goods or services; loss of use, data or profits;
 * or business interruption) however caused and on any theory of
 * liability, whether in contract, strict liability, or tort
 * (including negligence or otherwise) arising from the use of this
 * software, even if advised of the possibility of such damage.
 * Some jurisdictions may not allow disclaimers of implied warranties
 * and certain statements in the above disclaimer may not apply to
 * you as regards implied warranties; the other terms and conditions
 * remain enforceable notwithstanding. In some jurisdictions it is
 * not permitted to limit liability and therefore such limitations
 * may not apply to you.
 *
 * @copyright 2012 Talldude Networks, LLC.
 */

// make sure we are not being called directly
defined('APP_DIR') or exit();

/**
 * meta
 */
function jrSearch_meta()
{
    $_tmp = array(
        'name'        => 'Search',
        'url'         => 'search',
        'version'     => '1.3.7',
        'developer'   => 'The Jamroom Network, &copy;' . strftime('%Y'),
        'description' => 'Search for items in modules that have registered with the system',
        'category'    => 'listing',
        'requires'    => 'jrCore:5.1.0',
        'license'     => 'mpl'
    );
    return $_tmp;
}

/**
 * init
 */
function jrSearch_init()
{
    jrCore_register_module_feature('jrCore', 'css', 'jrSearch', 'jrSearch.css');
    jrCore_register_module_feature('jrCore', 'javascript', 'jrSearch', 'jrSearch.js');

    jrCore_register_event_listener('jrCore', 'db_search_params', 'jrSearch_db_search_params_listener');

    // Maintain our full text index
    jrCore_register_event_listener('jrCore', 'db_create_item', 'jrSearch_db_create_item_listener');
    jrCore_register_event_listener('jrCore', 'db_update_item', 'jrSearch_db_update_item_listener');
    jrCore_register_event_listener('jrCore', 'db_delete_item', 'jrSearch_db_delete_item_listener');

    // Our index creator/worker
    jrCore_register_queue_worker('jrSearch', 'search_index', 'jrSearch_search_index_worker', 0, 1);

    // Site Builder widgets
    jrCore_register_module_feature('jrSiteBuilder', 'widget', 'jrSearch', 'widget_search', 'Site Search');
    // jrCore_register_module_feature('jrSiteBuilder', 'widget', 'jrSearch', 'widget_top', 'Top Searches');

    return true;
}

//------------------------------------
// WIDGETS
//------------------------------------

/**
 * Display CONFIG screen for Widget
 * @param $_post array Post info
 * @param $_user array User array
 * @param $_conf array Global Config
 * @param $_wg array Widget info
 * @return bool
 */
function jrSearch_widget_search_config($_post, $_user, $_conf, $_wg)
{
    global $_mods;

    // module
    $_opt = jrCore_get_datastore_modules();
    foreach ($_opt as $mod => $url) {
        if(!jrCore_module_is_active($mod)){
            unset($_opt[$mod]);
            continue;
        }
        switch ($mod) {
            // Some modules we don't support or they support themselves
            case 'jrSeamless':
            case 'jrSmiley':
                unset($_opt[$mod]);
                break;

            default:
                if (is_file(APP_DIR . "/modules/{$mod}/templates/item_list.tpl")) {
                    $_opt[$mod] = $_mods[$mod]['module_name'];
                }
                else {
                    unset($_opt[$mod]);
                }
        }
    }
    $_opt['_'] = '- Search All Modules -';
    natcasesort($_opt);

    $_tmp = array(
        'name'     => 'search_module',
        'label'    => 'Search Module',
        'help'     => 'Select the module whos items you want to search',
        'options'  => $_opt,
        'value'    => (isset($_wg['widget_data']['search_module'])) ? $_wg['widget_data']['search_module'] : '_',
        'type'     => 'select',
        'validate' => 'printable'
    );
    jrCore_form_field_create($_tmp);

    return true;
}

/**
 * Get Widget results from posted Config data
 * @param $_post array Post info
 * @return bool
 */
function jrSearch_widget_search_config_save($_post)
{
    $_data = array(
        'search_module' => $_post['search_module']
    );
    return array('widget_data' => $_data);
}

/**
 * Widget DISPLAY
 * @param $_widget array Page Widget info
 * @return string
 */
function jrSearch_widget_search_display($_widget)
{
    return jrCore_parse_template('widget_search.tpl', $_widget, 'jrSearch');
}

//---------------------------------------------------------
// QUEUE WORKER
//---------------------------------------------------------

/**
 * Maintain a FULL TEXT index for a module
 * @param $_queue array Queue info
 * @return bool
 */
function jrSearch_search_index_worker($_queue)
{
    $mod = $_queue['module'];

    // Has the module told us what fields to index?
    $_fl = jrSearch_get_module_index_fields($mod);
    if ($_fl) {

        $key = '_item_id';
        switch ($mod) {
            case 'jrProfile':
                $key = '_profile_id';
                break;
            case 'jrUser':
                $key = '_user_id';
                break;
        }

        // Get items and add to index
        $off = 0;
        $_ky = array($key);
        $_ky = array_merge($_fl, $_ky);
        while (true) {

            $_rt = array(
                'search'        => array(
                    "_item_id > {$off}"
                ),
                'skip_triggers' => true,
                'return_keys'   => $_ky,
                'limit'         => 2500
            );
            $_rt = jrCore_db_search_items($mod, $_rt);
            if ($_rt && is_array($_rt) && isset($_rt['_items'])) {
                $_in = array();
                foreach ($_rt['_items'] as $_v) {
                    $off = (int) $_v[$key];
                    $txt = jrSearch_get_index_text_from_item($_v, $_fl);
                    if (strlen($txt) > 0) {
                        $_in[] = "('{$mod}',{$off},'" . jrCore_db_escape($txt) . "')";
                    }
                }
                if (count($_in) > 0) {
                    $tbl = jrCore_db_table_name('jrSearch', 'fulltext');
                    $req = "INSERT INTO {$tbl} (`s_module`, `s_id`, `s_text`) VALUES " . implode(',', $_in) . " ON DUPLICATE KEY UPDATE `s_text` = VALUES(`s_text`)";
                    jrCore_db_query($req);
                    usleep(250000);
                }
                if ($off < 2500) {
                    break;
                }
            }
            else {
                break;
            }
        }
    }
    return true;
}

//---------------------------------------------------------
// EVENT LISTENERS
//---------------------------------------------------------

/**
 * Add Text fields to full text search
 * @param $_data array Array of information from trigger
 * @param $_user array Current user
 * @param $_conf array Global Config
 * @param $_args array additional parameters passed in by trigger caller
 * @param $event string Triggered Event name
 * @return array
 */
function jrSearch_db_create_item_listener($_data, $_user, $_conf, $_args, $event)
{
    // Get our index fields
    if (isset($_conf['jrSearch_fulltext']) && $_conf['jrSearch_fulltext'] == 'on') {
        $_fl = jrSearch_get_module_index_fields($_args['module']);
        if ($_fl) {
            $txt = jrSearch_get_index_text_from_item($_data, $_fl);
            $txt = jrCore_db_escape($txt);
            $tbl = jrCore_db_table_name('jrSearch', 'fulltext');
            $req = "INSERT INTO {$tbl} (`s_module`, `s_id`, `s_text`) VALUES ('{$_args['module']}', '{$_args['_item_id']}', '{$txt}') ON DUPLICATE KEY UPDATE `s_text` = '{$txt}'";
            jrCore_db_query($req);
        }
    }
    return $_data;
}

/**
 * Update Text fields for full text search
 * @param $_data array Array of information from trigger
 * @param $_user array Current user
 * @param $_conf array Global Config
 * @param $_args array additional parameters passed in by trigger caller
 * @param $event string Triggered Event name
 * @return array
 */
function jrSearch_db_update_item_listener($_data, $_user, $_conf, $_args, $event)
{
    // Get our index fields
    if (isset($_conf['jrSearch_fulltext']) && $_conf['jrSearch_fulltext'] == 'on') {
        $_fl = jrSearch_get_module_index_fields($_args['module']);
        if ($_fl) {
            $_it = jrCore_db_get_item($_args['module'], $_args['_item_id'], true);
            if ($_it && is_array($_it)) {
                $_it = array_merge($_it, $_data);
                $txt = jrSearch_get_index_text_from_item($_it, $_fl);
                $txt = jrCore_db_escape($txt);
                $tbl = jrCore_db_table_name('jrSearch', 'fulltext');
                $req = "INSERT INTO {$tbl} (`s_module`, `s_id`, `s_text`) VALUES ('{$_args['module']}', '{$_args['_item_id']}', '{$txt}') ON DUPLICATE KEY UPDATE `s_text` = '{$txt}'";
                jrCore_db_query($req);
            }
        }
    }
    return $_data;
}

/**
 * Delete an entry from the fulltext index
 * @param $_data array Array of information from trigger
 * @param $_user array Current user
 * @param $_conf array Global Config
 * @param $_args array additional parameters passed in by trigger caller
 * @param $event string Triggered Event name
 * @return array
 */
function jrSearch_db_delete_item_listener($_data, $_user, $_conf, $_args, $event)
{
    if (isset($_conf['jrSearch_fulltext']) && $_conf['jrSearch_fulltext'] == 'on') {
        $tbl = jrCore_db_table_name('jrSearch', 'fulltext');
        $req = "DELETE FROM {$tbl} WHERE `s_module` = '{$_args['module']}' AND `s_id` = '{$_args['_item_id']}'";
        jrCore_db_query($req);
    }
    return $_data;
}

/**
 * Add support for custom list parameters
 * @param $_data array Array of information from trigger
 * @param $_user array Current user
 * @param $_conf array Global Config
 * @param $_args array additional parameters passed in by trigger caller
 * @param $event string Triggered Event name
 * @return array
 */
function jrSearch_db_search_params_listener($_data, $_user, $_conf, $_args, $event)
{
    global $_urls, $_post;
    if (isset($_post['module_url']) && isset($_post['ss']) && strlen($_post['ss']) > 0 && isset($_SESSION['jr-search-fields'])) {

        // See if this is from a profile or the site
        if (!isset($_urls["{$_post['module_url']}"])) {
            // Profile...
            $pfx = jrCore_db_get_prefix($_urls["{$_post['option']}"]);
            $mod = $_urls["{$_post['option']}"];
        }
        else {
            // Module
            $pfx = jrCore_db_get_prefix($_post['module']);
            $mod = $_post['module'];
        }
        if ($pfx) {

            // See we if have full text searching enabled
            $ft = false;
            if (isset($_conf['jrSearch_fulltext']) && $_conf['jrSearch_fulltext'] == 'on') {

                // See what the minimum word length is for using Full Text Searching (default for MySQL is 4)
                $len = 4;
                $req = "SHOW VARIABLES LIKE 'ft_min_word_len'";
                $_ln = jrCore_db_query($req, 'SINGLE');
                if ($_ln && isset($_ln['Value'])) {
                    $len = (int) $_ln['Value'];
                }
                if (strlen($_post['ss']) >= $len) {

                    $ft = true;
                    // Are we doing natural language or boolean?
                    // http://dev.mysql.com/doc/refman/5.1/en/fulltext-boolean.html
                    // http://dev.mysql.com/doc/refman/5.1/en/fulltext-natural-language.html
                    $nat = true;
                    $_mt = explode(' ', $_post['ss']);
                    if (strpos($_post['ss'], 'AND') || strpos($_post['ss'], 'OR')) {
                        // Clean ANDs
                        foreach ($_mt AS $k => $word) {
                            if ($word == 'AND') {
                                $prv = ($k - 1);
                                if (isset($_mt[$prv])) {
                                    $_mt[$prv] = '+' . str_replace('+', '', $_mt[$prv]);
                                }
                                $nxt = ($k + 1);
                                if (isset($_mt[$nxt])) {
                                    $_mt[$nxt] = '+' . str_replace('+', '', $_mt[$nxt]);
                                }
                                unset($_mt[$k]);
                            }
                            elseif ($word == 'OR') {
                                unset($_mt[$k]);
                            }
                        }
                        $_post['ss'] = implode(' ', $_mt);
                    }

                    $_mt = explode(' ', $_post['ss']);
                    if ($_mt && is_array($_mt)) {
                        foreach ($_mt as $str) {
                            $char = substr($str, 0, 1);
                            switch ($char) {
                                case '+';
                                case '-';
                                case '"';
                                case '~';
                                    $nat = false;
                                    break 2;
                            }
                            if (strpos($str, '*')) {
                                $nat = false;
                                break;
                            }
                        }
                    }

                    $tbl = jrCore_db_table_name('jrSearch', 'fulltext');
                    if ($nat) {
                        // Natural Language
                        $req = "SELECT `s_id` FROM {$tbl} WHERE `s_module` = '" . jrCore_db_escape($mod) . "' AND MATCH (`s_text`) AGAINST ('" . jrCore_db_escape($_post['ss']) . "' IN NATURAL LANGUAGE MODE)";
                    }
                    else {
                        // We're doing BOOLEAN
                        $req = "SELECT `s_id`, MATCH (`s_text`) AGAINST ('" . jrCore_db_escape($_post['ss']) . "' IN BOOLEAN MODE) AS score FROM {$tbl}
                             WHERE `s_module` = '" . jrCore_db_escape($mod) . "' AND MATCH (`s_text`) AGAINST ('" . jrCore_db_escape($_post['ss']) . "' IN BOOLEAN MODE) ORDER BY score DESC";
                    }
                    $_rt = jrCore_db_query($req, 's_id', false, 's_id');

                    // Did we get our results based on a full text search?
                    if ($_rt && is_array($_rt)) {

                        $key = '_item_id';
                        switch ($mod) {
                            case 'jrProfile':
                                $key = '_profile_id';
                                break;
                            case 'jrUser':
                                $key = '_user_id';
                                break;
                        }

                        // We are adding a search condition
                        if (!isset($_data['search'])) {
                            $_data['search'] = array();
                        }
                        $_data['search'][] = "{$key} in " . implode(',', $_rt);
                    }
                    else {
                        // We had NO RESULTS - set impossible match
                        $_data['search'] = array('_item_id < 0');
                    }
                }
            }

            // "normal" search (like)
            if (!$ft) {
                switch ($_SESSION['jr-search-fields']) {

                    case 'all':
                        $_data['search'] = "{$pfx}_% like %{$_post['ss']}%";
                        break;

                    default:
                        $_fld = explode(',', $_SESSION['jr-search-fields']);
                        if ($_fld && is_array($_fld)) {
                            $_tmp = array();
                            foreach ($_fld as $field) {
                                if (strpos($field, $pfx) === 0) {
                                    $_tmp[] = "{$field} like %{$_post['ss']}%";
                                }
                            }
                            if ($_tmp && count($_tmp) > 0) {
                                $_data['search'][] = implode(' || ', $_tmp);
                            }
                        }
                        break;
                }
            }

            // Next - see if we have any additional fields that were passed in by function
            if (isset($_SESSION['jrSearch_search_params']) && is_array($_SESSION['jrSearch_search_params'])) {
                foreach ($_SESSION['jrSearch_search_params'] as $cond) {
                    $_data['search'][] = $cond;
                }
                unset($_SESSION['jrSearch_search_params']);
            }

        }

    }
    return $_data;
}

//---------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------

/**
 * Get the index fields for a given module
 * @param $module string Module to get fields for
 * @return array|bool
 */
function jrSearch_get_module_index_fields($module)
{
    global $_conf;
    // Has the module told us what fields to index?
    $_fl = array();
    $_tm = jrCore_get_registered_module_features('jrSearch', 'fulltext_search_fields');
    if (isset($_tm[$module])) {
        $tmp = array_keys($_tm[$module]);
        $tmp = reset($tmp);
        $_fl = explode(',', $tmp);
    }
    else {
        // Figure it out
        $pfx = jrCore_db_get_prefix($module);
        $_fn = array(
            "{$pfx}_name"        => 1,
            "{$pfx}_title"       => 1,
            "{$pfx}_text"        => 1,
            "{$pfx}_desc"        => 1,
            "{$pfx}_description" => 1,
            "{$pfx}_caption"     => 1,
            "{$pfx}_tags"        => 1
        );
        // See if this module has registered any search fields
        $_tm = jrCore_get_registered_module_features('jrSearch', 'search_fields');
        if (isset($_tm[$module])) {
            $tmp = array_keys($_tm[$module]);
            $tmp = reset($tmp);
            $_fl = explode(',', $tmp);
        }
        $tbl = jrCore_db_table_name($module, 'item_key');
        $req = "SELECT `key` FROM {$tbl} GROUP BY `key`";
        $_rt = jrCore_db_query($req, 'key');
        if ($_rt && is_array($_rt)) {
            foreach ($_fn as $fld => $i) {
                if (isset($_rt[$fld]) && !in_array($fld, $_fl)) {
                    $_fl[] = $fld;
                }
            }
        }
    }
    // See if there are any custom fields for our module
    $pfx = jrCore_db_get_prefix($module);
    if (strpos(' ' . $_conf['jrSearch_search_fields'], $pfx)) {
        foreach (explode("\n", $_conf['jrSearch_search_fields']) as $v) {
            $v = trim($v);
            if (strpos($v, "{$pfx}_") === 0) {
                $_fl[] = $v;
            }
        }
    }
    return (count($_fl) > 0) ? $_fl : false;
}

/**
 * Get our index text from an item and fields
 * @param $_item array Item Array
 * @param $_fields array fields to get text for
 * @return string
 */
function jrSearch_get_index_text_from_item($_item, $_fields)
{
    $txt = '';
    foreach ($_fields as $fld) {
        if (isset($_item[$fld])) {
            if (strpos($fld, '_tags')) {
                $txt .= trim(str_replace(',', ' ', $_item[$fld])) . ' ';
            }
            else {
                $txt .= strip_tags($_item[$fld]) . ' ';
            }
        }
    }
    return rtrim($txt);
}

/**
 * Get DataStore modules we can enable FULL TEXT searching on
 * @return array
 */
function jrSearch_get_search_modules()
{
    // See what modules are listening for us
    global $_mods;
    $_ot = array();
    $_tm = jrCore_get_registered_module_features('jrSearch', 'fulltext_search_fields');
    if ($_tm && is_array($_tm)) {
        foreach ($_tm as $module => $_inf) {
            if (isset($_mods[$module]['module_name'])) {
                $_ot[$module] = $_mods[$module]['module_name'];
            }
        }
    }
    return (count($_ot) > 0) ? $_ot : false;
}

//---------------------------------------------------------
// SMARTY FUNCTIONS
//---------------------------------------------------------

/**
 * Build a search form
 * @param $params array parameters for function
 * @param $smarty object Smarty object
 * @return string
 */
function smarty_function_jrSearch_form($params, $smarty)
{
    global $_conf;

    // In: module="ModuleName" or module="all" for a global search (default: all)
    // In: page (default:1)
    // In: pagebreak (default:10)
    // In: template (default: html_search_form.tpl)
    // In: class (optional)
    // In: style (optional)
    // In: assign (optional)

    if (!jrCore_module_is_active('jrSearch')) {
        return '';
    }

    // Check the incoming parameters
    if (empty($params['module'])) {
        $params['module'] = 'all';
    }
    if (!isset($params['page']) || !jrCore_checktype($params['page'], 'number_nz')) {
        $params['page'] = 1;
    }

    if (!isset($params['pagebreak']) || !jrCore_checktype($params['pagebreak'], 'number_nz')) {
        $params['pagebreak'] = (isset($_conf['jrSearch_index_limit'])) ? intval($_conf['jrSearch_index_limit']) : 4;
    }

    if (!isset($params['value'])) {
        $_lang           = jrUser_load_lang_strings();
        $params['value'] = $_lang['jrSearch'][1];
    }

    if (empty($params['style'])) {
        $params['style'] = '';
    }

    if (empty($params['class'])) {
        $params['class'] = '';
    }

    if (!empty($params['template'])) {
        $params['tpl_dir'] = $_conf['jrCore_active_skin'];
    }
    else {
        $params['template'] = 'html_search_form.tpl';
        $params['tpl_dir']  = 'jrSearch';
    }
    if (!isset($params['method'])) {
        $params['method'] = 'post';
    }
    $_tmp = array();
    foreach ($params as $k => $v) {
        $_tmp['jrSearch'][$k] = $v;
    }

    // Call the appropriate template and return
    $out = jrCore_parse_template($params['template'], $_tmp, $params['tpl_dir']);
    if (isset($params['assign']) && strlen($params['assign']) > 0) {
        $smarty->assign($params['assign'], $out);
        return '';
    }
    return $out;
}

/**
 * jrSearch_recent
 * Show most recent searches
 * @param $params array parameters for function
 * @param $smarty object Smarty object
 * @return string
 */
function smarty_function_jrSearch_recent($params, $smarty)
{
    global $_conf;
    if (!jrCore_module_is_active('jrSearch')) {
        return '';
    }
    // Check the incoming parameters
    $_s = array();
    if (isset($params['user_id']) && jrCore_checktype($params['user_id'], 'number_nz')) {
        $_s[] = "_user_id = {$params['user_id']}";
    }
    if (isset($params['module']) && $params['module'] != '') {
        $_s[] = "search_module = {$params['module']}";
    }
    if (!isset($params['limit']) || jrCore_checktype($params['limit'], 'number_nz') || $params['limit'] > 100) {
        $params['limit'] = 5;
    }
    if (!isset($params['style']) || strlen($params['style']) === 0) {
        $params['style'] = '';
    }
    if (!isset($params['class']) || strlen($params['class']) === 0) {
        $params['class'] = '';
    }
    if (isset($params['template']) && $params['template'] != '') {
        $params['tpl_dir'] = $_conf['jrCore_active_skin'];
    }
    else {
        $params['template'] = "search_recent.tpl";
        $params['tpl_dir']  = 'jrSearch';
    }

    $_tmp = array();
    foreach ($params as $k => $v) {
        $_tmp['jrSearch'][$k] = $v;
    }

    // Get most recent
    $_s  = array(
        'search'        => $_s,
        'order_by'      => array("_created" => "desc"),
        'return_keys'   => array('search_module', 'search_string'),
        'skip_triggers' => true,
        'limit'         => $params['limit']
    );
    $_rt = jrCore_db_search_items('jrSearch', $_s);
    if ($_rt && is_array($_rt) && is_array($_rt['_items'])) {
        foreach ($_rt['_items'] as $k => $rt) {
            $_tmp['jrSearchRecent'][$k]['module'] = $rt['search_module'];
            $_tmp['jrSearchRecent'][$k]['string'] = jrCore_entity_string($rt['search_string']);
        }
    }

    $out = jrCore_parse_template($params['template'], $_tmp, $params['tpl_dir']);
    if (isset($params['assign']) && $params['assign'] != '') {
        $smarty->assign($params['assign'], $out);
        return '';
    }
    return $out;
}

/**
 * Show most popular searches
 * @param $params array parameters for function
 * @param $smarty object Smarty object
 * @return string
 */
function smarty_function_jrSearch_popular($params, $smarty)
{
    global $_conf;
    if (!jrCore_module_is_active('jrSearch')) {
        return '';
    }
    // Check the incoming parameters
    if (!isset($params['limit']) || !jrCore_checktype($params['limit'], 'number_nz') || $params['limit'] > 100) {
        $params['limit'] = 5;
    }
    if (!isset($params['style']) || strlen($params['style']) === 0) {
        $params['style'] = '';
    }
    if (!isset($params['class']) || strlen($params['class']) === 0) {
        $params['class'] = '';
    }
    if (isset($params['template']) && $params['template'] != '') {
        $params['tpl_dir'] = $_conf['jrCore_active_skin'];
    }
    else {
        $params['template'] = "search_popular.tpl";
        $params['tpl_dir']  = 'jrSearch';
    }
    $_tmp = array();
    foreach ($params as $k => $v) {
        $_tmp['jrSearch'][$k] = $v;
    }

    // Get most popular
    $tbl = jrCore_db_table_name('jrSearch', 'item_key');
    $req = "SELECT `_item_id`, COUNT(*) AS cnt FROM {$tbl} WHERE `key` = 'search_string' GROUP BY `value` ORDER BY `count` DESC LIMIT {$params['limit']}";
    $_rt = jrCore_db_query($req, 'NUMERIC');
    if (isset($_rt[0]) && is_array($_rt[0])) {
        foreach ($_rt as $k => $rt) {
            $_item                                 = jrCore_db_get_item('jrSearch', $rt['_item_id']);
            $_tmp['jrSearchPopular'][$k]['module'] = $_item['search_module'];
            $_tmp['jrSearchPopular'][$k]['string'] = jrCore_entity_string($_item['search_string']);
            $_tmp['jrSearchPopular'][$k]['count']  = (int) $rt['cnt'];
        }
    }

    $out = jrCore_parse_template($params['template'], $_tmp, $params['tpl_dir']);
    if (isset($params['assign']) && strlen($params['assign']) > 0) {
        $smarty->assign($params['assign'], $out);
        return '';
    }
    return $out;
}

/**
 * Show search area for a module index
 * @param $params array parameters for function
 * @param $smarty object Smarty object
 * @return string
 */
function smarty_function_jrSearch_module_form($params, $smarty)
{
    global $_urls, $_post;
    if (!isset($params['module'])) {
        if (!isset($_post['module'])) {
            jrCore_smarty_missing_error('module');
        }
        $params['module'] = $_post['module'];
    }
    if (!isset($params['template'])) {
        $params['template'] = 'search_module_form.tpl';
    }
    if (!isset($params['fields']) || strlen($params['fields']) === 0) {
        $params['fields'] = 'all';
    }
    if (!isset($_post['ss']) || strlen($_post['ss']) === 0) {
        $_SESSION['jr-search-fields'] = $params['fields'];
    }

    // We can get additional params for our jrCore_db_search_items
    $_args = array();
    foreach ($params as $k => $v) {
        // Search
        if (strpos($k, 'search') === 0) {
            $_args[] = $v;
        }
    }
    if (count($_args) > 0) {
        $_SESSION['jrSearch_search_params'] = $_args;
    }

    // See if we are on a SITE module index (index.tpl) OR
    // on a profile module index (item_index.tpl)
    if (isset($_post['module_url'])) {
        if (isset($_post['module_url']) && !isset($_urls["{$_post['module_url']}"])) {
            // We're on a profile...
            $params['search_url'] = "{$_post['module_url']}/{$_post['option']}";
        }
        else {
            $params['search_url'] = $_post['module_url'];
            // See if we have additional options
            $_check = array('option', '_1', '_2');
            foreach ($_check as $chk) {
                if (isset($_post[$chk]) && strlen($_post[$chk]) > 0) {
                    $params['search_url'] .= "/{$_post[$chk]}";
                }
            }
        }
    }
    $out = jrCore_parse_template($params['template'], $params, 'jrSearch');
    if (!empty($params['assign'])) {
        $smarty->assign($params['assign'], $out);
        return '';
    }
    return $out;
}
